!
! ml 15/11/99
!
! This program uses the parameters and data structures defined in
! edusoft_mod and makes a call to all the routines included in
! edusoft_mod, emutils_mod and badpixutils_mod
! (except sizeListParam and readListParam).
 

PROGRAM check

  use emutils_mod
  use edusoft_mod
  use badpixutils_mod
  use cal
  use dal
  use param
  use errorhandling

! For edusoft

  type(edu_evt)     :: eduevt
  type(edu_pattern) :: edupat(edu_npat)
  type(edu_out)     :: eduout

  integer, parameter :: dx = 100, dy = 100
  integer :: x0 =  10, y0 =  10 
  integer :: edumode = 3  ! Image mode
  integer :: ii, jj
  integer(kind=int16), dimension(0:1023)         :: offX
  integer(kind=int16), dimension(0:1023)         :: offY  
  integer(kind=int8),  dimension(:,:,:), pointer :: patterns
  integer(kind=int16)                            :: eduThreshold
  integer(kind=int16), dimension(dx,dy)          :: im

  integer(kind=int16), dimension(-1:1,-1:1,0:edu_npat-1) :: patabove
  integer(kind=int16), dimension(0:edu_npat-1)           :: pixelInE2
  integer,             dimension(-1:1)                   :: locX, locY
  type(sas_evt), dimension(1)  :: evt
  integer(kind=int32), dimension(-2:10,-2:10)        :: imagein

! For emutils

  type(TableT)      :: ev_tab
  type(DataSetT)    :: ev_set
  character(len=80) :: filename = "../emodf/0001_183536AIV-_M1S00170IME.FTZ"

  logical           :: imag, timi, redI, comp
  character(len=*), parameter :: taskname = "test_emsaplib"
  character(len=8), dimension(9), parameter :: keycompat = &
         (/"TELESCOP","INSTRUME","CCDID   ","CCDNODE ",&
           "WINDOWX0","WINDOWY0","WINDOWDX","WINDOWDY","FRMTIME "/)

! For badpixutils

  integer,parameter :: maxbad = 32768
  character(len=80) :: tempfile = "toto.out"
  integer, dimension(-10:EMOS_MAX_X+20)          :: badoffX
  integer, dimension(-10:EMOS_MAX_Y+20)          :: badoffY
  integer(kind=int16), dimension(:), allocatable :: xbad,ybad,yext,tbad,fbad
  integer :: nbad, k, kmax
  real(double) :: pon
  real, dimension(6) :: mu, eps
  real(double), dimension(1000) :: cdf

  eduevt%x = 4

  call message(AppMsg,Verbose,"PARAMETER VALUES :")

  write(errstr,*) "edu_npat = ",edu_npat
  call message(AppMsg,Verbose)

  write(errstr,*) "edu_nsid = ",edu_nsid
  call message(AppMsg,Verbose)

  write(errstr,*) "es_nmax  = ",es_nmax   
  call message(AppMsg,Verbose)

! Read the pattern library

  call CAL_getEventPatterns(patterns, eduThreshold)

  call pat_init(patterns,edupat)  ! Initialize EDUSOFT

  eduThreshold = 10
    
  do ii = 0, 1023 
     offX( ii ) = 0
     offY( ii ) = 0
  enddo
  do jj = 1, dy
     do ii = 1, dx
        im( ii, jj ) = 0
     enddo
  enddo  

  im( 20, 10 ) = 100

  call edusoft(edumode,edupat,dx,dy,im,eduThreshold,  &
                   offX(0:x0+dx-1),offY(0:y0+dy-1),x0,y0, &
                   es_nmax,eduout)
 
  write(errstr,*) "EDUSOFT OUTPUT :"
  call message(AppMsg,Verbose)
  write(errstr,*) "eduout%nevent = ",eduout%nevent
  call message(AppMsg,Verbose)
  if( eduout%nevent > 0 ) then
     write(errstr,*) "eduout%evt(1)%x = ",eduout%evt(1)%x   
     call message(AppMsg,Verbose)
     write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%y   
     call message(AppMsg,Verbose)
     write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%e1
     call message(AppMsg,Verbose)
     write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%e2
     call message(AppMsg,Verbose)
  endif

! CALL pixelInE2 and inMask

  call getpixelInE2(pixelInE2, patabove)
  locX = (/0,0,1/)
  locY = (/1,0,0/)
  write(errstr,*) "Offsets X:", locX
  call message(AppMsg,Verbose)
  write(errstr,*) "Offsets Y:", locY
  call message(AppMsg,Verbose)
  write(errstr,*) "Pattern 10:", pixelInE2(10), " pixels in E2. ",  &
                  "Offset projection:", inMask(locX, locY, patabove(:,:,10))
  call message(AppMsg,Verbose)

! CALL projectEventsCounts
! evt        : Array of event structure
! patabove   : Geometry of event outside central pixel
! image      : Image upon which to project

  evt(1)%rawx = 5
  evt(1)%rawy = 5
  evt(1)%pattern = 1
  evt(1)%e1   = 1000
  evt(1)%e2   = 50
 
  call message(AppMsg,Verbose," call  projectEventsCounts()")
  call projectEventsCounts(evt, patabove, imagein)

! CALL projectEventsEnergy

  call message(AppMsg,Verbose," call  projectEventsEnergy()")
  call projectEventsEnergy(evt, patabove, imagein)

! CALL  addFilename 

  ev_set = dataSet(filename, Modify)
  ev_tab = table(ev_set, 0)  ! Extension name depends on mode

  call message(AppMsg,Verbose," CALL addFilename")
  call addFilename(block(ev_tab), filename,"zozo")

! CALL  changeCase 

  write(errstr,*) changeCase("call CHANGEcASE",0)
  call message(AppMsg,Verbose)

! CALL  getCcd 

  write(errstr,*) "CCD NUMBER =", getCcd(block(ev_tab))
  call message(AppMsg,Verbose)

! CALL  getMode 

  call message(AppMsg,Verbose," CALL getMode") 
  call getMode(ev_tab, imag, timi, redI, comp)
  write(errstr,*) "imaging=", imag," timing=",timi," redImaging=", &
                  redI, " compTiming=",comp
  call message(AppMsg,Verbose)

! CALL  keywordDone 

  call message(AppMsg,Verbose," CALL keywordDone")  
  call keywordDone(block(ev_tab), taskname, "ZOZO")

  write(errstr,*) "Was it done ? :", wasDone(block(ev_tab), "ZOZO" )
  call message(AppMsg,Verbose)

! CALL  keywordRemove

  call message(AppMsg,Verbose," CALL keywordRemove")  
  call keywordRemove(block(ev_tab), "ZOZO" )

! CALL  wasDone 

  call message(AppMsg,Verbose," CALL keywordDone")  
  write(errstr,*) "Was it done ? :",wasDone(block(ev_tab), "ZOZO" )
  call message(AppMsg,Verbose)

! CALL  equalKeywords 

  call message(AppMsg,Verbose," CALL equalKeywords")  
  write(errstr,*) "Compat. ? :", equalKeywords(attributable(ev_tab), &
                  attributable(ev_set), keycompat,.false.)
  call message(AppMsg,Verbose)

! CALL  readBadOffsets

  call readBadOffsets(ev_set, badoffX, badoffY, 3)
  call release(ev_set)

! These calls need a parameter list, which we haven't here
! See evlistcomb_mod.f90 for examples of how to call.
! CALL  sizeListParam 
! CALL  readListParam

! Calls for bad pixels

  allocate(xbad(maxbad),ybad(maxbad),yext(maxbad),tbad(maxbad),fbad(maxbad))
  ev_set = dataSet(tempfile, Temp)
  ev_tab = addTable(ev_set, "BADPIX", 0)

  call message(AppMsg,Verbose," CALL readBadpix")  
  call readBadpix(ev_tab, .false., xbad, ybad, yext, tbad, fbad, nbad)

  nbad = 1
  xbad(1) = 12
  ybad(1) = 257
  yext(1) = 5
  tbad(1) = 1
  fbad(1) = 3

  call message(AppMsg,Verbose," CALL mergeBad")  
  call mergeBad(xbad, ybad, yext, tbad, fbad, nbad)

  call message(AppMsg,Verbose," CALL writeBadpix")  
  call writeBadpix(ev_tab, xbad, ybad, yext, tbad, fbad, nbad)

  call release(ev_set)
  deallocate(xbad,ybad,yext,tbad,fbad)

! Test quantilePoisson and cumulativePoisson function in all conditions
! Gaussian approximation starts at 300

  mu = (/0.5,299.9999,300.0001,0.5,299.9999,300.0001/)
  eps = (/1.e-20,1.e-20,1.e-20,-1.e-6,-1.e-6,-1.e-6/)
  do k = 1,3
     kmax = int(mu(k)) + 3*int(sqrt(mu(k)))
     call cumulativePoisson(real(mu(k),kind=double),kmax,kmax+200,cdf)
     write(errstr,*) "mu =", mu(k), "       Poisson CDF =", cdf(1:7)
     call message(AppMsg,Verbose)
     call compCumulPoisson(real(mu(k),kind=double),kmax,kmax+200,cdf)
     write(errstr,*) "mu =", mu(k), "  comp Poisson CDF =", cdf(1:7)
     call message(AppMsg,Verbose)
  enddo
  do k = 1,6
     kmax = quantilePoisson(mu(k),eps(k))
     write(errstr,*) "mu =", mu(k), "  epsilon =", eps(k),  &
                     "  quantile =", kmax
     call message(AppMsg,Verbose)
  enddo

! Test cumulativeBinomial

  pon = 0.04d0
  kmax = 10
  do k = 0,6
     cdf(k+1) = cumulativeBinomial(kmax,k,pon)
  enddo
  write(errstr,*) "Pon =", pon, "  Non =", kmax, "Noff = 0 to 6"
  call message(AppMsg,Verbose)
  write(errstr,*) "Binomial CDF =", cdf(1:7)
  call message(AppMsg,Verbose)

! Test median computation for integer and real array

  write(errstr,*) "median of mu =", localMedian(mu)
  call message(AppMsg,Verbose)
  write(errstr,*) "median of int(mu(1:5)) =", localMedian(int(mu),5)
  call message(AppMsg,Verbose)

END PROGRAM check
